import requests

from hudai import __version__, HudAiError
from hudai.resources import *

USER_AGENT = 'HUD.ai Python v{} +(https://github.com/FoundryAI/hud-ai-python#readme)'.format(__version__)

class HudAi:
    def __init__(self, api_key=None, base_url='https://api.hud.ai'):
        if not api_key:
            raise HudAiError('missing api_key', 'initialization_error')

        self._api_key = api_key
        self._base_url = base_url

        self.article_company = ArticleCompanyResource(self)
        self.article_highlights = ArticleHighlightsResource(self)
        self.article_key_term = ArticleKeyTermResource(self)
        self.clean_article = CleanArticleResource(self)
        self.company = CompanyResource(self)
        self.domain = DomainResource(self)
        self.key_term = KeyTermResource(self)
        self.message = MessageResource(self)
        self.news_api_article = NewsApiArticleResource(self)
        self.participant = ParticipantResource(self)
        self.relevant_article = RelevantArticleResource(self)
        self.rss_article = RssArticleResource(self)
        self.rss_feed_metadata = RssFeedMetadataResource(self)
        self.task = TaskResource(self)
        self.text_corpus = TextCorpusResource(self)
        self.user = UserResource(self)


    def get(self, path, params={}, data={}):
        return requests.get(self._build_url(path),
                            params=params,
                            data=data,
                            headers=self._get_headers())


    def post(self, path, params={}, data={}):
        return requests.post(self._build_url(path),
                            params=params,
                            data=data,
                            headers=self._get_headers())


    def put(self, path, params={}, data={}):
        return requests.put(self._build_url(path),
                            params=params,
                            data=data,
                            headers=self._get_headers())


    def patch(self, path, params={}, data={}):
        return requests.patch(self._build_url(path),
                            params=params,
                            data=data,
                            headers=self._get_headers())


    def delete(self, path, params={}, data={}):
        return requests.delete(self._build_url(path),
                            params=params,
                            data=data,
                            headers=self._get_headers())


    def _build_url(self, path):
        return '{}{}'.format(self._base_url, path)


    def _get_headers(self):
        return { 'User-Agent': USER_AGENT, 'x-api-key': self._api_key }



"""
Python wrapper for HUD.ai data access layer

Example usage:

from hudai.client import HudAi

client = HudAi()
"""

__version__ = '1.4.2'

class HudAiError(Exception):
    def __init__(self, message=None, type='validation_error'):
        super(HudAiError, self).__init__(message)

        self._message = message
        self.type = type



from hudai.resource import Resource
from pydash import pick


class UserResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, '/users')
        self.resource_name = 'User'

    def get(self, id):
        return self.request({
            'method': 'GET',
            'params': {'id': id},
            'url': '/internal/{id}'
        })

    def list(self, params):
        return self.request({
            'method': 'GET',
            'params': pick(params, 'email', 'name'),
            'url': '/internal'
        })

    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params, 'email', 'name', 'hash'),
            'url': '/internal'
        })

    def update(self, params):
        return self.request({
            'method': 'PUT',
            'data': pick(params, 'email', 'name', 'hash'),
            'params': pick(params, 'id'),
            'url': '/internal/{id}'
        })

    def delete(self, id):
        return self.request({
            'method': 'DELETE',
            'params': {'id': id},
            'url': '/internal/{id}'
        })



from hudai.resources.article_company import ArticleCompanyResource
from hudai.resources.article_highlights import ArticleHighlightsResource
from hudai.resources.article_key_term import ArticleKeyTermResource
from hudai.resources.clean_article import CleanArticleResource
from hudai.resources.company import CompanyResource
from hudai.resources.domain import DomainResource
from hudai.resources.key_term import KeyTermResource
from hudai.resources.message import MessageResource
from hudai.resources.news_api_article import NewsApiArticleResource
from hudai.resources.participant import ParticipantResource
from hudai.resources.relevant_article import RelevantArticleResource
from hudai.resources.rss_article import RssArticleResource
from hudai.resources.rss_feed_metadata import RssFeedMetadataResource
from hudai.resources.task import TaskResource
from hudai.resources.text_corpus import TextCorpusResource
from hudai.resources.user import UserResource



from hudai import HudAiError

class Resource(object):
    def __init__(self, client, base_path=''):
        """
        :param client: API client
        """

        if client is None:
            raise HudAiError('client required', 'initialization_error')

        self._client = client
        self._base_path = base_path


    def request(self, params):
        """
        Abstracted request method, request config is defined in the resource itself
        :param params:
        :return:
        """
        method = params.get('method', 'get').lower()
        query_params = params.get('params', {})
        url = params.get('url', '')
        data = params.get('data', {})

        path = self._build_path(url, query_params)

        if method == 'get':
            response = self._client.get(path, params=query_params, data=data)

        elif method == 'post':
            response = self._client.post(path, params=query_params, data=data)

        elif method == 'put':
            response = self._client.put(path, params=query_params, data=data)

        elif method == 'patch':
            response = self._client.patch(path, params=query_params, data=data)

        elif method == 'delete':
            response = self._client.delete(path, params=query_params, data=data)

        else:
            raise ValueError('method.invalid:{}'.format(method))

        return response.json()


    def _build_path(self, url, query_params):
        """
        Build the url path string
        :return url:
        """
        path = "{}{}".format(self._base_path, url)

        if query_params is None:
            return path

        return path.format(**query_params)



import requests

from hudai.client import HudAi
from hudai.resources import *


TEST_PATH = '/test/url'
TEST_PARAMS = {'foo':'bar'}
TEST_DATA = {'fizz':'buzz'}


def test_initialization():
    client = HudAi(api_key='mock-api-key')

    assert isinstance(client, HudAi)
    assert isinstance(client.article_company, ArticleCompanyResource)
    assert isinstance(client.article_highlights, ArticleHighlightsResource)
    assert isinstance(client.article_key_term, ArticleKeyTermResource)
    assert isinstance(client.clean_article, CleanArticleResource)
    assert isinstance(client.company, CompanyResource)
    assert isinstance(client.domain, DomainResource)
    assert isinstance(client.message, MessageResource)
    assert isinstance(client.news_api_article, NewsApiArticleResource)
    assert isinstance(client.participant, ParticipantResource)
    assert isinstance(client.relevant_article, RelevantArticleResource)
    assert isinstance(client.rss_article, RssArticleResource)
    assert isinstance(client.rss_feed_metadata, RssFeedMetadataResource)
    assert isinstance(client.task, TaskResource)
    assert isinstance(client.text_corpus, TextCorpusResource)
    assert isinstance(client.user, UserResource)


def test_get(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.get')

    assert callable(client.get)

    client.get(TEST_PATH, params=TEST_PARAMS, data=TEST_DATA)

    assert requests.get.call_count == 1

    args, kwargs = requests.get.call_args

    assert args[0] == 'https://api.hud.ai{}'.format(TEST_PATH)
    assert kwargs['params'] == TEST_PARAMS
    assert kwargs['data'] == TEST_DATA
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'


def test_post(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.post')

    assert callable(client.post)

    client.post(TEST_PATH, params=TEST_PARAMS, data=TEST_DATA)

    assert requests.post.call_count == 1

    args, kwargs = requests.post.call_args

    assert args[0] == 'https://api.hud.ai{}'.format(TEST_PATH)
    assert kwargs['params'] == TEST_PARAMS
    assert kwargs['data'] == TEST_DATA
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'


def test_put(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.put')

    assert callable(client.put)

    client.put(TEST_PATH, params=TEST_PARAMS, data=TEST_DATA)

    assert requests.put.call_count == 1

    args, kwargs = requests.put.call_args

    assert args[0] == 'https://api.hud.ai{}'.format(TEST_PATH)
    assert kwargs['params'] == TEST_PARAMS
    assert kwargs['data'] == TEST_DATA
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'


def test_patch(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.patch')

    assert callable(client.patch)

    client.patch(TEST_PATH, params=TEST_PARAMS, data=TEST_DATA)

    assert requests.patch.call_count == 1

    args, kwargs = requests.patch.call_args

    assert args[0] == 'https://api.hud.ai{}'.format(TEST_PATH)
    assert kwargs['params'] == TEST_PARAMS
    assert kwargs['data'] == TEST_DATA
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'


def test_delete(mocker):
    client = HudAi(api_key='mock-api-key')
    mocker.patch('requests.delete')

    assert callable(client.delete)

    client.delete(TEST_PATH, params=TEST_PARAMS, data=TEST_DATA)

    assert requests.delete.call_count == 1

    args, kwargs = requests.delete.call_args

    assert args[0] == 'https://api.hud.ai{}'.format(TEST_PATH)
    assert kwargs['params'] == TEST_PARAMS
    assert kwargs['data'] == TEST_DATA
    assert kwargs['headers']['x-api-key'] == 'mock-api-key'



from setuptools import setup, find_packages

from hudai import __version__

HOMEPAGE = 'https://github.com/FoundryAI/hud-ai-python'

setup(
  name = 'hudai',
  packages = find_packages(),
  version = __version__,
  description = 'HUD.ai python bindings',
  author = 'HUD.ai Engineering',
  author_email = 'engineering@hud.ai',
  url = HOMEPAGE,
  download_url = '{}/releases/{}.tar.gz'.format(HOMEPAGE, __version__),
  keywords = ['hudai', 'foundry.ai', 'foundrydc'], # arbitrary keywords
  classifiers = [
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 3 - Alpha',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',

    # Pick your license as you wish (should match "license" above)
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7'
  ],
  install_requires=[
    'requests',
    'pydash'
  ]
)



from pytest_mock import mocker

from hudai.client import HudAi
from hudai.resource import Resource

client = HudAi(api_key='mock-api-key')

def test_inherited_resource_methods(mocker):
    resource = Resource(client)

    assert callable(resource.request)


def test_request(mocker):
    mocker.patch.object(client, 'get', autospec=True)
    resource = Resource(client)

    method = 'GET'
    params = { 'test': 'param' }
    data = { 'test': 'data' }
    url = '/test/path'

    resource.request({
        'method': method,
        'url': url,
        'params': params,
        'data': data
    })

    client.get.assert_called_once_with(url, params=params, data=data)


def test_request_defaults(mocker):
    mocker.patch.object(client, 'get', autospec=True)
    resource = Resource(client)

    resource.request({ 'url': '/test/path' })

    client.get.assert_called_once_with('/test/path', params={}, data={})


def test_request_path_building(mocker):
    mocker.patch.object(client, 'get', autospec=True)
    resource = Resource(client)

    params = { 'replace_me': 'foo' }

    resource.request({ 'url': '/test/{replace_me}/path', 'params': params })

    client.get.assert_called_once_with('/test/foo/path', params=params, data={})



from hudai.resource import Resource
from pydash import pick


class ArticleCompanyResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/article-companies')
        self.resource_name = 'ArticleCompany'

    def get(self, article_uuid):
        return self.request({
            'method': 'GET',
            'params': {'article_uuid': article_uuid},
            'url': '/internal/{article_uuid}'
        })

    def list(self, params):
        return self.request({
            'method': 'GET',
            'params': pick(params, 'company_id', 'article_type', 'published_at'),
            'url': '/internal'
        })

    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params, 'company_id', 'article_type', 'published_at'),
            'url': '/internal'
        })

    def update(self, params):
        return self.request({
            'method': 'PUT',
            'data': pick(params, 'company'),
            'params': pick(params, 'article_uuid'),
            'url': '/internal/{article_uuid}'
        })

    def delete(self, article_uuid):
        return self.request({
            'method': 'DELETE',
            'params': {'id': article_uuid},
            'url': '/internal/{article_uuid}'
        })



from hudai.resource import Resource
from pydash import pick


class TaskResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/tasks')
        self.resource_name = 'Task'

    def get(self, uuid):
        return self.request({
            'method': 'GET',
            'params': {'uuid': uuid},
            'url': '/internal/{uuid}'
        })

    def create(self, params):
        return self.request({
            'method': 'POST',
            'data': pick(params, 'uuid'),
            'url': '/internal'
        })

    def delete(self, uuid):
        return self.request({
            'method': 'DELETE',
            'params': {'uuid': uuid},
            'url': '/internal/{uuid}'
        })
